import pytest

from muvisdk.SDK import SDK

# Credenciales de test
merchant = {
    '_id': '012345',
    'name': 'Un Merchant de test',
    'credentials': {
        'mercadopago': {
            'access_token': 'TEST-7306398834844755-071810-f54f60f3cb6c0da2bb76c3201711f7fa-1161352268'
        },
        'decidir': {
            'public_key': 'fEvHoakBPLO2vMkFHpCNwt5H7F1albtY',
            'access_token': '7UEr3kX4paS91dXwZw2H4avATuN6jNSt'
        }
    }
}


def mercadopago_params():
    merchant['credentials']['preferred_processor'] = 'mercadopago'
    return (
        SDK(merchant),
        '1163536800-CLBbg46pSRduJi',  # customer_id
        '1658246890412',  # card_id
        '9734d788ac10cbc7bc8b1a73060575b8'  # card_token
    )


def decidir_params():
    merchant['credentials']['preferred_processor'] = 'decidir'
    return (
        SDK(merchant),
        '588b945d-9333-4a8c-af76-95412eb02913',  # customer_id
        '40973830e2f4f625f96705fa7b60c446d951bc4c518ebea5af2d4225b43bf3d6',  # card_id
        'caf63949-f149-445f-bdb0-f838a2f9691e'  # card_token
    )


@pytest.mark.parametrize(
    ('sdk', 'customer_id', 'card_id', 'card_token'),
    [
        mercadopago_params(),
        decidir_params()
    ]
)
class TestCard:
    def test_create_card(self, sdk, customer_id, card_id, card_token):
        r = sdk.card().create(customer_id, card_token)
        assert r['status'] == 'ok'
        assert r['response']['id'] is not None

    def test_get_card_found(self, sdk, customer_id, card_id, card_token):
        r = sdk.card().get(customer_id, card_id)
        assert r['status'] == 'ok'
        assert r['response']['id'] is not None
        assert r['response']['id'] == card_id



import os

from .decidir_sdk.DecidirSDK import DecidirSDK
from .mercadopago_sdk.MercadoPagoSDK import MercadoPagoSDK


class SDK:
    def __init__(
        self,
        merchant: dict,
        merchant_cadena: dict,
        processor: str = None,
        marketplace: bool = False,
        _test = None,
        force: bool = False
    ):
        self._sdk = None

        # Se verifica la variable de entorno:
        if _test is not None: 
            test_sdk = _test
        else:
            test_env = os.getenv('TEST')
            test_sdk = True
            if test_env.lower() == 'false':
                test_sdk = False

        # Se verifica el procesor
        if processor == 'mercadopago':
            self.processor = 'mercadopago'
            self._sdk = MercadoPagoSDK(merchant, marketplace=marketplace)
        elif processor == 'decidir':
            self.processor = 'decidir'
            self._sdk = DecidirSDK(
                merchant,
                merchant_cadena,
                marketplace=marketplace,
                test=test_sdk,
                force=force
            )

    def customer(self):
        return self._sdk.customer()

    def card(self):
        return self._sdk.card()

    def card_token(self):
        return self._sdk.card_token()

    def payment(self):
        return self._sdk.payment()

    def refund(self):
        return self._sdk.refund()

    def ok(self):
        return self._sdk is not None and self._sdk.ok()



import mercadopago

from ..MuviBase import MuviBase


def _format(response: dict) -> dict:
    return {
        'email': response['email'],
        'first_name': response['first_name'],
        'last_name': response['last_name'],
        'id': response['id']
    }


class Customer(MuviBase):
    def __init__(self, processor: str, sdk: mercadopago.SDK):
        super().__init__(processor)
        self.sdk = sdk

    def _validate_client(self, client: dict) -> dict:
        fields = ['nombre', 'apellido', 'documento', 'email', 'domicilio']
        for f in fields:
            if f not in client:
                return self.error(message='{} field is missing'.format(f))

        for f in ['calle', 'altura']:
            if f not in client['domicilio']:
                return self.error(message='domicilio.{} field is missing'.format(f))

        return self.ok(response=client)

    def create(self, client: dict) -> dict:
        """

        :param client: Diccionario del cliente
        nombre, apellido, documento, email, domicilio son obligatorios
        :return:
        """
        # Check para el client y evitar errores
        client = client.copy()
        r = self._validate_client(client)
        if r['status'] >= 400:
            return r

        customer_response = self.sdk.customer().search(filters={'email': client['email']})
        if len(customer_response['response']['results']) > 0:
            return self.ok(response=_format(customer_response['response']['results'][0]))

        try:
            street_number = int(client['domicilio']['altura'])
        except:
            street_number = 1

        mp_customer_data = {
            'email': client['email'],
            'first_name': client['nombre'],
            'last_name': client['apellido'],
            'phone': {
                'area_code': None,
                'number': client['celular']
            },
            'identification': {
                'type': 'DNI',
                'number': client['documento']
            },  # ! Tipo de identificacion hardcodeado
            'address': {
                'street_name': client['domicilio']['calle'],
                'street_number': street_number,
            }
        }
        mp_response = self.sdk.customer().create(mp_customer_data)
        if mp_response['status'] < 400:
            return self.ok(response=_format(mp_response['response']), status=mp_response['status'])

        return self.error(response=mp_response['response'], status=mp_response['status'])

    def get(self, customer_id: str):
        mp_response = self.sdk.customer().get(customer_id)
        if mp_response['status'] < 400:
            customer = mp_response['response']
            return self.ok(response=_format(customer), status=mp_response['status'])

        return self.error(response=mp_response['response'], status=mp_response['status'])

    def search(self, filters: dict):
        mp_response = self.sdk.customer().search(filters)
        if mp_response['status'] < 400:
            return self.ok(response=mp_response['response'], status=mp_response['status'])

        return self.error(response=mp_response['response'], status=mp_response['status'])



from .Card import Card
from .CardToken import CardToken
from .Customer import Customer
from .Payment import Payment
from .Refund import Refund


class DecidirSDK:
    def __init__(
        self,
        merchant: dict,
        merchant_cadena: dict,
        marketplace: bool = False,
        test: bool = True,
        force: bool = False
    ):
        if (decidir := merchant.get('credentials', {}).get('decidir', None)):
            if not decidir.get('site_id', None):
                self.private_key = None
                return
            
            if not force:
                if not decidir.get('active', False):
                    self.private_key = None
                    return
        
        credentials_cadena = merchant_cadena.get('credentials', {}).get('decidir', {})
        credentials_local = merchant.get('credentials', {}).get('decidir', {})

        environment = 'developers' if test else 'live'
        self.url = f'https://{environment}.decidir.com/api/v2'

        self.private_key = credentials_cadena.get('access_token')
        self.public_key = credentials_cadena.get('public_key')

        self.merchant_name = merchant.get('name')

        self.site_id = credentials_local.get('site_id')
        self.site_id_cadena_con_cvv = credentials_cadena.get('site_id_con_cvv')
        self.site_id_cadena_sin_cvv = credentials_cadena.get('site_id')

        self.processor = 'decidir'
        self.marketplace = marketplace

    def customer(self):
        return Customer(self.processor, self.url, self.private_key, self.public_key)
    
    def card(self):
        return Card(self.processor, self.url, self.private_key, self.public_key)

    def card_token(self):
        return CardToken(self.processor, self.url, self.private_key, self.public_key)

    def payment(self):
        return Payment(
            self.processor,
            self.url,
            self.private_key,
            self.public_key,
            self.merchant_name,
            self.site_id,
            self.site_id_cadena_con_cvv,
            self.site_id_cadena_sin_cvv,
            self.marketplace
        )

    def refund(self):
        return Refund(
            self.processor,
            self.url,
            self.private_key,
            self.public_key,
            self.merchant_name,
            self.site_id,
            self.site_id_cadena_con_cvv,
            self.site_id_cadena_sin_cvv,
            self.marketplace
        )

    def ok(self):
        return self.private_key is not None



class MuviBase:
    def __init__(self, processor):
        self.processor = processor

    def ok(self, response, status=200):
        response['processor'] = self.processor
        return {
            'status': status,
            'response': response
        }

    def error(self, response: dict = None, message: str = None, status: int = 400):
        if not response:
            response = {}
        if message:
            response['message'] = message
        if 'status' not in response:
            response['status'] = status
        response['processor'] = self.processor
        return {
            'status': status,
            'response': response
        }



import mercadopago
import requests
from mercadopago.http import HttpClient as MpHttpClient
from requests.adapters import HTTPAdapter
from urllib3.util import Retry

from .Card import Card
from .CardToken import CardToken
from .Customer import Customer
from .Payment import Payment
from .Refund import Refund


class HttpClient(MpHttpClient):

    def request(self, method, url, maxretries=None, **kwargs):
        retry_strategy = Retry(
            total=maxretries,
            status_forcelist=[429, 500, 502, 503, 504]
        )
        http = requests.Session()
        http.mount("https://", HTTPAdapter(max_retries=retry_strategy))
        with http as session:
            api_result = session.request(method, url, **kwargs)
            response = {
                "status": api_result.status_code,
                "response": api_result.json(),
                "headers": api_result.headers
            }

        return response


class MercadoPagoSDK:
    def __init__(self, merchant: dict, marketplace: bool = False):
        mercadopago_model = 'mp_marketplace' if marketplace else 'mercadopago'

        active = merchant['credentials'][mercadopago_model]['active']
        access_token = merchant['credentials'][mercadopago_model]['access_token'] if active else None

        self.sdk = mercadopago.SDK(access_token, http_client=HttpClient()) if active else None
        self.merchant_id = merchant['_id']
        self.merchant_name = merchant['name']
        self.processor = 'mercadopago'

    def customer(self):
        return Customer(self.processor, self.sdk)
    
    def card(self):
        return Card(self.processor, self.sdk)

    def card_token(self):
        return CardToken(self.processor, self.sdk)
    
    def payment(self):
        return Payment(self.processor, self.sdk, self.merchant_id)

    def refund(self):
        return Refund(self.processor, self.sdk)

    def ok(self):
        return self.sdk is not None



import mercadopago

from ..MuviBase import MuviBase


class Refund(MuviBase):
    def __init__(self, processor: str, sdk: mercadopago.SDK):
        super().__init__(processor)
        self.sdk = sdk

    def create(self, payment_id: str, amount: float = None):
        refund_data = {
            "amount": amount
        }
        refund_response = self.sdk.refund().create(payment_id, refund_data)
        if refund_response['status'] < 400:
            return self.ok(response=refund_response['response'], status=refund_response['status'])
        else:
            return self.error(response=refund_response['response'], status=refund_response['status'])



import pytest
import random
import string

from muvisdk.SDK import SDK

# Credenciales de test
merchant = {
    '_id': '012345',
    'name': 'Un Merchant de test',
    'credentials': {
        'mercadopago': {
            'access_token': 'TEST-7306398834844755-071810-f54f60f3cb6c0da2bb76c3201711f7fa-1161352268'
        },
        'decidir': {
            'public_key': 'fEvHoakBPLO2vMkFHpCNwt5H7F1albtY',
            'access_token': '7UEr3kX4paS91dXwZw2H4avATuN6jNSt'
        }
    }
}


def mercadopago_params():
    merchant['credentials']['preferred_processor'] = 'mercadopago'
    return (
        SDK(merchant)
    )


def decidir_params():
    merchant['credentials']['preferred_processor'] = 'decidir'
    return (
        SDK(merchant)
    )


client = {
    'email': 'francisco006@muvinai.com',
    'nombre': 'Francisco',
    'apellido': 'Viñas',
    'celular': '11111111',
    'documento': '22222222',
    'domicilio': {
        'altura': '123',
        'calle': 'Calle Falsa'
    }
}


def random_string(length):
    letters = string.ascii_lowercase
    return ''.join(random.choice(letters) for _ in range(length))


def random_email():
    return '{}@{}.com'.format(random_string(10), random_string(5))


@pytest.mark.parametrize('sdk', [mercadopago_params(), decidir_params()])
class TestCustomer:
    def test_get_customer_not_found(self, sdk):
        r = sdk.customer().get('128763198273698172')
        assert r['status'] == 'error'

    def test_create_and_get_customer(self, sdk):
        new_client = client.copy()
        new_client['email'] = random_email()
        r = sdk.customer().create(new_client)
        assert r['status'] == 'ok'
        create_response = r['response']
        assert create_response['email'] == new_client['email']
        assert create_response['first_name'] == new_client['nombre']
        assert create_response['last_name'] == new_client['apellido']
        assert create_response['id'] is not None

        r = sdk.customer().get(create_response['id'])
        assert r['status'] == 'ok'
        get_response = r['response']
        assert get_response['email'] == new_client['email']
        assert get_response['first_name'] == new_client['nombre']
        assert get_response['last_name'] == new_client['apellido']
        assert get_response['id'] == create_response['id']
